@@ -7,6 +7,9 @@ use crate::lexer::LexerRule;
 use crate::lexer::NextLexeme;
 use std::rc::Rc;
 
+/// Utility for creating [lexer rules](LexerRule).
+///
+/// Please read the [crate documentation](crate) for more information and examples.
 pub struct LexerBuilder {
     table: Vec<LexerRule>,
 }
@@ -18,10 +21,16 @@ impl Default for LexerBuilder {
 }
 
 impl LexerBuilder {
+    /// Creates a new [LexerBuilder] with no rules.
     pub fn new() -> LexerBuilder {
         LexerBuilder { table: Vec::new() }
     }
 
+    /// Add a rule that will be active
+    /// when the current [Lexer] state matches any of `states`,
+    /// with name `name`,
+    /// that matches exactly the content of `string`,
+    /// and performs the provided `action`.
     pub fn string(
         &mut self,
         states: &[&str],
@@ -45,6 +54,11 @@ impl LexerBuilder {
         self
     }
 
+    /// Add a rule that will be active
+    /// when the current [Lexer] state matches any of `states`,
+    /// with name `name`,
+    /// that matches the regular expression `pattern`,
+    /// and performs the provided `action`.
     #[cfg(feature = "crate_regex")]
     pub fn pattern(
         &mut self,
@@ -54,12 +68,12 @@ impl LexerBuilder {
         action: fn(&mut Lexer) -> NextLexeme,
     ) -> &mut LexerBuilder {
         let regex =
-            crate_regex::Regex::new(&format!("^(?:{pattern})")).unwrap();
+            crate_regex::Regex::new(&format!(r"\A(?:{pattern})")).unwrap();
 
         self.table.push(LexerRule {
             action:  Rc::new(action),
             matcher: Rc::new(move |input: &str| -> Option<usize> {
-                regex.find(input).map(|match_| match_.end())
+                regex.find_iter(input).take(1).map(|match_| match_.end()).next()
             }),
             name:    name.to_string(),
             states:  states.iter().map(|state| state.to_string()).collect(),
@@ -68,6 +82,7 @@ impl LexerBuilder {
         self
     }
 
+    /// Return the created [LexerRule]s.
     pub fn finish(&self) -> Vec<LexerRule> {
         self.table.clone()
     }
